import router from "./index"
import store from "../store/index"
import { Message } from 'element-ui'

//import {ucenterRouter} from "./ucenter_router"

//  路由导航守卫
router.beforeEach(async(to, from, next) => {

    let authenticated = store.getters.isAuthenticated;

    if(authenticated){
      console.log("已认证");

      //登录页面直接进入默认主页
      if(to.path === "/login"){
        next({path:'/'});
        return ;
      }

      //如权限加载直接进入页面
      let isLoadPermit = store.getters.isLoadPermit;
      if (isLoadPermit){
        //加载菜单信息
        let tags = store.getters.menuTags.filter(function(menuTag){
          return menuTag.fullPath === to.fullPath;
        });
        let titleName = tags.length>0? tags[0].meta.title : to.meta.title;
        to.meta.title = titleName;
        //跳转页面
        next();
        return ;
      }
     
      try{
        //加载权限及模块资源
        //加载用户信息
        await store.dispatch('user/userInfo');
        //加载角色
        await store.dispatch('permit/userRoles');
        //加载权限
        await store.dispatch('permit/userPerms');
        //动态加载资源
        const newRoutes = await store.dispatch('router/newRoutes');
        console.log(newRoutes);
        router.addRoutes(newRoutes);
        //加载页签
        store.commit('tagsView/ADD_ROUTES_TAGS', router.getRoutes());
        //跳转
        next({ ...to, replace: true })
      }catch(error){
        //重新登录
        await store.dispatch('user/resetToken');
        Message.error(error || 'Has Error');
        next(`/login?redirect=${to.path}`);
      }
    }else{
      console.log("未认证");
      //受保护
      if (to.path.startsWith("/private")||to.path.startsWith("/protected")){
        next(`/login?redirect=${to.path}`);
      }else{
        next();
        return;
      }
    }
})

