
import { getToken,setToken,removeToken } from '../../utils/auth'
import { oauthClient } from '../../oauth_request.js'
import { OAuthPasswordVerifyCode } from '../../../src/gcloud/oauth/oauth_pb'
import { StringValue } from 'gcloud-core-vue/src/core/util/basic_wrappers.js'
import { oauthUserClient } from '../../oauth_user_request'
import { Empty } from 'google-protobuf/google/protobuf/empty_pb'

const state = {
  token: getToken(),
  user: {},
  refresh : false
}

const mutations = {
  SET_TOKEN: (state, token) => {
    state.token = token
  },
  SET_USER: (state, user) => {
    state.user = user
  },
  SET_REFRESH: (state, refresh) => {
    state.refresh = refresh
  }
}

const actions = {
    //用户登录
    login({commit},loginForm){
      return new Promise((resolve, reject) => {
        let oAuthPasswordVerifyCode = new OAuthPasswordVerifyCode();
        oAuthPasswordVerifyCode.setUsername(loginForm.username);
        oAuthPasswordVerifyCode.setPassword(loginForm.password);
        oAuthPasswordVerifyCode.setVerifyCode(loginForm.verificationCode)
        oAuthPasswordVerifyCode.setVerifyCodeId(loginForm.verifyId)
        oauthClient.authorizePwdVerify(oAuthPasswordVerifyCode).then((response) =>{
          const timestamp = (new Date()).valueOf();
          const token =response.toObject();
          token.authTime=timestamp;
          console.log(token);
          
          commit('SET_TOKEN', token);
          setToken(state.token);
          resolve(state.token);
        }).catch(err => {
          reject(err);
        })
      });
    },
    //用户登出
    logout({commit}){
      return new Promise((resolve) => {
        commit('SET_TOKEN', {});
        commit('SET_USER', {});
        removeToken();
        //resetRouter()
        //dispatch('tagsView/delAllViews', null, { root: true })
        resolve();
      });
    },
    // 重置认证
    resetToken({ commit }) {
      return new Promise(resolve => {
        commit('SET_TOKEN', {});
        commit('SET_USER', {});
        removeToken()
        resolve()
      })
    },
    //刷新Token
    refreshToken({commit,state}){
      return new Promise((resolve, reject) => {
          let token = state.token;
          if (Object.keys(token).length == 0){
            reject('请先登录!');
            return;
          }
          oauthClient.refreshToken(StringValue.of(token.refreshToken))
          .then(response =>{
              const timestamp = (new Date()).valueOf();
              const token =response.toObject();
              token.authTime=timestamp;
              console.log(token);
              
              commit('SET_TOKEN', token);
              setToken(state.token);
              resolve(state.token);
          }).catch(err=>{
            reject(err);
          });
      });
    },
    //获取最新AccessToken
   accessToken({state,commit,dispatch}){
      return new Promise((resolve, reject) => {
          let token = state.token;
          if (Object.keys(token).length == 0){
              reject('请先登录!');
              return;
          }
          const timestamp = (new Date()).valueOf();
          const expiryTime =  token.authTime + token.expiresIn *1000 - 300000;
          // const expiryTime = token.authTime + token.expiresIn *1000 - 300000
          if(timestamp<expiryTime || state.refresh){
            resolve(state.token.accessToken);
          }else{
            commit('SET_REFRESH', true);
            dispatch('refreshToken').then(() => {
              resolve(state.token.accessToken);
              commit('SET_REFRESH', false);
            }).catch(err=>{
              console.log(err);
              resolve(state.token.accessToken);
              commit('SET_REFRESH', false);
            });
          }
      });
    },
    //用户信息获取
    userInfo({ commit }){
      return new Promise((resolve, reject) => {
        let empty = new Empty
        oauthUserClient.userInfo(empty)
        .then(response =>{
          const user =response.toObject();
          commit('SET_USER', user);
          resolve(user);
        })
        .catch(err =>{
          reject(err);
        });
      });
    }
}

export default {
  namespaced: true,
  state,
  mutations,
  actions
}
